"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pluginsLoadersProvidersAndListeners = void 0;
const core_1 = require("../core");
const providers_loader_1 = require("./providers.loader");
/**
 * @internal
 * Register all listeners before the plugins are loaded to be sure that the scope middleware has already been created.
 * Also register all providers that are typically created in plugins.
 * @param app Express app
 */
async function pluginsLoadersProvidersAndListeners(app, providers) {
    const pluginLoader = await Promise.resolve().then(() => require('@medusajs/medusa/dist/loaders/plugins'));
    const originalPluginLoader = pluginLoader.default;
    pluginLoader.default = async (cradle) => {
        app.use(async (req, res, next) => {
            req.container = cradle.container;
            await core_1.customEventEmitter.registerListeners(req.scope);
            return next();
        });
        await originalPluginLoader(cradle);
        await (0, providers_loader_1.registerProviders)(cradle.container, providers, cradle.configModule);
    };
}
exports.pluginsLoadersProvidersAndListeners = pluginsLoadersProvidersAndListeners;
//# sourceMappingURL=plugins.loader.js.map