"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerFulfillmentProvider = exports.registerTaxProvider = exports.registerProvider = exports.registerProviders = void 0;
const awilix_1 = require("awilix");
const core_1 = require("../core");
const logger = core_1.Logger.contextualize('ProvidersLoader');
function registerProviders(container, providers, configModule) {
    for (const providerOptions of providers) {
        registerProvider(container, providerOptions, configModule);
    }
    logger.log(`${providers.length} providers registered`);
}
exports.registerProviders = registerProviders;
function registerProvider(container, options, configModule) {
    const { subtype } = options;
    if (subtype === 'tax')
        return registerTaxProvider(container, options, configModule);
    if (subtype === 'fulfillment')
        return registerFulfillmentProvider(container, options, configModule);
}
exports.registerProvider = registerProvider;
function registerTaxProvider(container, options, configModule) {
    const { metatype } = options;
    const name = (0, core_1.lowerCaseFirst)(metatype.name);
    const identifier = ensureIdentifier(metatype);
    console.log('container', container);
    container.registerAdd('taxProviders', (0, awilix_1.asFunction)((cradle) => new metatype(cradle, configModule.options)));
    container.register({
        [name]: (0, awilix_1.asFunction)((cradle) => new metatype(cradle, configModule.options)).singleton(),
        [`tp_${identifier}`]: (0, awilix_1.aliasTo)(name),
    });
    logger.log(`Tax Provider loaded - ${name}`);
}
exports.registerTaxProvider = registerTaxProvider;
function registerFulfillmentProvider(container, options, configModule) {
    const { metatype } = options;
    const name = (0, core_1.lowerCaseFirst)(metatype.name);
    const identifier = ensureIdentifier(metatype);
    container.registerAdd('fulfillmentProviders', (0, awilix_1.asFunction)((cradle) => new metatype(cradle, configModule.options)));
    container.register({
        [name]: (0, awilix_1.asFunction)((cradle) => new metatype(cradle, configModule.options)).singleton(),
        [`fp_${identifier}`]: (0, awilix_1.aliasTo)(name),
    });
    logger.log(`Fulfillment Provider loaded - ${name}`);
}
exports.registerFulfillmentProvider = registerFulfillmentProvider;
function ensureIdentifier(metatype) {
    if (!metatype.identifier)
        throw new Error(`${metatype.name} is missing a static identifier property`);
    return metatype.identifier;
}
//# sourceMappingURL=providers.loader.js.map